import os
from enum import Enum
from functools import wraps

from src.core.entity.ResponseTypes import ResponseTypes
from src.core.use_case import (
	clients_use_case,
	users_use_case,
)
from src.infra.api.fastapi.tools import (
	get_auth,
)


class Permission(Enum):
	CAN_READ_USERS = 'can_read_users'
	CAN_EDIT_USER_DATA = 'can_edit_user_data'
	CAN_EDIT_USER_PASSWORD = 'can_edit_user_password'
	CAN_DELETE_USERS = 'can_delete_users'


def get_access_token(access_token: str) -> dict:
	auth = get_auth()
	try:
		token_introspect = auth.token_introspect(
			params={
				'access_token': access_token,
				'app_name': os.environ.get('APP_NAME'),
			}
		)
		return {
			'type': ResponseTypes.SUCCESS,
			'data': token_introspect['data'],
		}
	except Exception as e:
		return {
			'type': ResponseTypes.ACCESS_DENIED,
			'errors': [{'field': 'access_token', 'message': str(e)}],
			'data': None,
		}


def auth_required(func):
	@wraps(func)
	async def wrapper(*args, **kwargs):
		if os.environ.get('API_DISABLE_AUTH', 'False').lower() != 'true':
			access_token = kwargs['request'].headers.get('Authorization', None)
			if not access_token:
				return {
					'type': ResponseTypes.ACCESS_DENIED,
					'errors': [
						{
							'field': 'access_token',
							'message': 'Missing access token',
						}
					],
					'data': None,
				}
			access_token = access_token.split(' ')[1]
			access_token_data = get_access_token(access_token)
			if access_token_data.get('data', {}).get('active', False) is False:
				return {
					'type': ResponseTypes.ACCESS_DENIED,
					'errors': [
						{
							'field': 'access_token',
							'message': 'Access token is not active. Check that the domain is correct and the token is not expired.',
						}
					],
					'data': None,
				}
			if access_token_data['type'] == ResponseTypes.SUCCESS:
				kwargs['access_token_data'] = access_token_data['data']
			else:
				kwargs['access_token_data'] = None
		return await func(*args, **kwargs)

	return wrapper


def check_permissions(permissions: list[str]):
	def decorator(func):
		@wraps(func)
		async def wrapper(*args, **kwargs):
			if os.environ.get('API_DISABLE_AUTH', 'False').lower() != 'true':
				access_token_data = kwargs['access_token_data']
				auth = get_auth()

				group_name = access_token_data.get('group', [None])[0]
				user_data = users_use_case.get(
					auth=auth,
					params={
						'app_name': os.environ.get('APP_NAME'),
						'client_id': clients_use_case.get_client_by_name(
							auth=auth,
							params={
								'app_name': os.environ.get('APP_NAME'),
								'client_name': group_name,
							},
						)['data']['id'],
						'user_id': access_token_data['sub'],
					},
				)
				if user_data['type'] != ResponseTypes.SUCCESS:
					return {
						'type': ResponseTypes.ACCESS_DENIED,
						'errors': [
							{
								'field': 'check permissions',
								'message': 'We could not find the user associated with the access token when checking permissions',
							}
						],
						'data': None,
					}
				user_permissions = [
					permission['name']
					for permission in user_data['data']['role']['permissions']
				]
				for permission in permissions:
					if permission not in user_permissions:
						return {
							'type': ResponseTypes.ACCESS_DENIED,
							'errors': [
								{
									'field': 'permissions',
									'message': 'You do not have permission to access this resource',
								}
							],
							'data': None,
						}
			return await func(*args, **kwargs)

		return wrapper

	return decorator
