from starlette.middleware.base import BaseHTTPMiddleware
from starlette.requests import Request

from src.core.entity.ResponseTypes import ResponseTypes
from src.infra.api.fastapi.tools import (
    convert_all_to_camel_case,
    convert_keys_to_camel_case,
    get_body,
    set_body,
)
from fastapi import status


class CorrectHTTPCodeMiddleware(BaseHTTPMiddleware):
    async def dispatch(self, request: Request, call_next):
        """
        Adjust the HTTP code based on the response type

        There will be delay and large use of RAM due to the size of the files that are returned.
        Source: https://stackoverflow.com/questions/69670125/how-to-log-raw-http-request-response-in-python-fastapi/73464007#73464007
        If the files are too large, consider using a different approach. For example, saving the files in a temporary folder and returning the path to the client or a decorator.
        """
        response = await call_next(request)
        response_json = await get_body(response)
        if response_json:
            match response_json.get("type", None):
                case ResponseTypes.PARAMETERS_ERROR:
                    response.status_code = status.HTTP_400_BAD_REQUEST
                case ResponseTypes.ACCESS_DENIED:
                    response.status_code = status.HTTP_401_UNAUTHORIZED
                case ResponseTypes.RESOURCE_ERROR:
                    response.status_code = status.HTTP_503_SERVICE_UNAVAILABLE
                case ResponseTypes.SYSTEM_ERROR:
                    response.status_code = status.HTTP_500_INTERNAL_SERVER_ERROR
                case _:
                    response.status_code = status.HTTP_200_OK
        else:
            response.status_code = status.HTTP_200_OK
        return response


class ResponseToCamelCase(BaseHTTPMiddleware):
    async def dispatch(self, request: Request, call_next):
        """
        Convert snake_case to camelCase
        """
        response = await call_next(request)
        response_json = await get_body(response)
        if response_json:
            response_json = convert_keys_to_camel_case(response_json)
            return set_body(response, response_json)
        return response


class ErrorsToCamelCase(BaseHTTPMiddleware):
    async def dispatch(self, request: Request, call_next):
        """
        Convert error from snake_case to camelCase
        """
        response = await call_next(request)
        response_json = await get_body(response)
        if (
            response_json
            and "type" in response_json
            and response_json["type"] != ResponseTypes.SUCCESS
        ):
            response_json["errors"] = convert_all_to_camel_case(response_json["errors"])
            return set_body(response, response_json)
        return response
