import json
from typing import Any

from starlette.concurrency import iterate_in_threadpool

from fastapi import Response


def is_json(myjson):
    try:
        json.loads(myjson)
    except ValueError:
        return False
    return True


async def get_body(response):
    response_body = [chunk async for chunk in response.body_iterator]
    response.body_iterator = iterate_in_threadpool(iter(response_body))
    if not response_body:
        return None
    decode_str = response_body[0].decode()
    if is_json(decode_str):
        return json.loads(decode_str)
    return None


def set_body(response: Response, body: dict[str, Any]) -> Response:
    """
    Set the body of the response
    """
    response.body = json.dumps(body).encode("utf-8")
    response.headers["Content-Length"] = str(len(response.body))
    return Response(
        content=json.dumps(body).encode("utf-8"),
        status_code=response.status_code,
        headers=dict(response.headers),
        media_type=response.media_type,
    )


def to_camel_case(input_str) -> str:
    """
    Convert snake_case to camelCase
    """
    components = input_str.split("_")
    return components[0] + "".join(x.title() for x in components[1:])


def convert_keys_to_camel_case(item: dict[str, Any]) -> dict[str, Any]:
    """
    Convert the keys of the dictionary to camel case (snake_case to camelCase) in all levels
    """
    if isinstance(item, dict):
        new_dict = {}
        for k, v in item.items():
            new_key = to_camel_case(k)
            new_dict[new_key] = convert_keys_to_camel_case(v)
        return new_dict
    elif isinstance(item, list):
        return [convert_keys_to_camel_case(item) for item in item]
    else:
        return item


def convert_all_to_camel_case(item: dict[str, Any]) -> dict[str, Any]:
    """
    Convert all keys to camel case (snake_case to camelCase) in all levels
    """
    if isinstance(item, dict):
        new_dict = {}
        for k, v in item.items():
            new_key = to_camel_case(k)
            new_value = to_camel_case(v) if isinstance(v, str) else v
            new_dict[new_key] = convert_all_to_camel_case(new_value)
        return new_dict
    elif isinstance(item, list):
        return [convert_all_to_camel_case(item) for item in item]
    else:
        return item


def to_snake_case(input_str) -> str:
    """
    Convert camelCase to snake_case
    """
    return "".join(["_" + i.lower() if i.isupper() else i for i in input_str]).lstrip(
        "_"
    )


def convert_keys_to_snake_case(item: dict[str, Any]) -> dict[str, Any]:
    """
    Convert the keys of the dictionary to snake case (camelCase to snake_case) in all levels
    """
    if isinstance(item, dict):
        new_dict = {}
        for k, v in item.items():
            new_key = to_snake_case(k)
            new_dict[new_key] = v
        return new_dict
    elif isinstance(item, list):
        return [convert_keys_to_snake_case(item) for item in item]
    else:
        return item
