import json
from flask import request
from typing import Any


def to_snake_case(input_str) -> str:
    """
    Convert camelCase to snake_case
    """
    return "".join(["_" + i.lower() if i.isupper() else i for i in input_str]).lstrip(
        "_"
    )


def to_camel_case(input_str) -> str:
    """
    Convert snake_case to camelCase
    """
    components = input_str.split("_")
    return components[0] + "".join(x.title() for x in components[1:])


def convert_keys_to_snake_case(item: dict[str, Any]) -> dict[str, Any]:
    """
    Convert the keys of the dictionary to snake case (camelCase to snake_case) in all levels
    """
    if isinstance(item, dict):
        new_dict = {}
        for k, v in item.items():
            new_key = to_snake_case(k)
            new_dict[new_key] = v
        return new_dict
    elif isinstance(item, list):
        return [convert_keys_to_snake_case(item) for item in item]
    else:
        return item


def convert_keys_to_camel_case(item: dict[str, Any]) -> dict[str, Any]:
    """
    Convert the keys of the dictionary to camel case (snake_case to camelCase) in all levels
    """
    if isinstance(item, dict):
        new_dict = {}
        for k, v in item.items():
            new_key = to_camel_case(k)
            new_dict[new_key] = convert_keys_to_camel_case(v)
        return new_dict
    elif isinstance(item, list):
        return [convert_keys_to_camel_case(item) for item in item]
    else:
        return item


def register_middlewares(app):
    """
    Registra los middlewares a la instancia de Flask
    """

    @app.before_request
    def convert_request_keys_to_snake_case_middleware():
        """
        Convert the keys of the request to snake case (camelCase to snake_case) in all levels
        """
        request.args = convert_keys_to_snake_case(request.args)

    @app.after_request
    def convert_response_to_camel_case_middleware(response):
        """
        Convert the keys of the response to camel case (snake_case to camelCase) in all levels
        """
        response.data = json.dumps(
            convert_keys_to_camel_case(json.loads(response.data))
        )
        return response

    @app.after_request
    def convert_response_to_json_middleware(response):
        """
        Convert the response to JSON format
        """
        response.headers["Content-Type"] = "application/json"
        return response

    @app.after_request
    def add_cors_headers_middleware(response):
        """
        Add CORS headers to the response
        """
        response.headers["Access-Control-Allow-Origin"] = "*"
        response.headers["Access-Control-Allow-Methods"] = "GET, POST, PUT, DELETE"
        return response
